@@ -134,7 +134,7 @@ _bgp_get_system_info( papi_mdi_t *mdi )
 	strcpy( _papi_hwi_system_info.hw_info.vendor_string, "IBM" );
 	tmp = snprintf( _papi_hwi_system_info.hw_info.model_string,
 					sizeof _papi_hwi_system_info.hw_info.model_string,
-					"PVR=0x%4.4x:0x%4.4x",
+					"PVR=%#4.4x:%#4.4x",
 					( utmp >> ( 31 - 15 ) ) & 0xFFFF, ( utmp & 0xFFFF ) );
 
 	BGP_Personality_getLocationString( &bgp, chipID );
@@ -544,7 +544,7 @@ user_signal_handler( int signum, hwd_siginfo_t * siginfo, void *mycontext )
 	EventSetInfo_t *ESI;
 	ThreadInfo_t *thread = NULL;
 	int isHardware = 1;
-	caddr_t pc;
+	vptr_t pc;
 	_papi_hwi_context_t ctx;
 	BGP_UPC_Event_Id_t xEventId = 0;
 //  int thresh;
@@ -556,7 +556,7 @@ user_signal_handler( int signum, hwd_siginfo_t * siginfo, void *mycontext )
 	ctx.ucontext = ( ucontext_t * ) mycontext;
 
 	ucontext_t *context = ( ucontext_t * ) mycontext;
-	pc = ( caddr_t ) context->uc_mcontext.regs->nip;
+	pc = ( vptr_t ) context->uc_mcontext.regs->nip;
 	thread = _papi_hwi_lookup_thread( 0 );
 	//int cidx = (int) &thread;
 	ESI = thread->running_eventset[0];
